@@ -24,10 +24,12 @@
 #ifndef _OUTPUT_GSTREAMER_H
 #define _OUTPUT_GSTREAMER_H
 
-extern int output_gstreamer_init(int *argc, char ***argv);
-extern void output_set_uri(const char *uri);
-extern int output_play(void);
-extern int output_stop(void);
-extern int output_pause(void);
+int output_gstreamer_add_options(GOptionContext *ctx);
+int output_gstreamer_init(void);
+void output_set_uri(const char *uri);
+int output_play(void);
+int output_stop(void);
+int output_pause(void);
+int output_loop(void);
 
 #endif /*  _OUTPUT_GSTREAMER_H */
