@@ -21,6 +21,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #define _GNU_SOURCE
 
 #include <stdio.h>
@@ -36,6 +40,8 @@
 #include <sys/socket.h>
 #include <arpa/inet.h>
 
+#include "logging.h"
+
 #include "webserver.h"
 #include "upnp.h"
 
@@ -55,17 +61,14 @@ static const char *param_datatype_names[] = {
         [DATATYPE_UNKNOWN] =    NULL
 };
 
-static void add_value_attribute(IXML_Document *doc, IXML_Element *parent, char *attrname, char *value)
+static void add_value_attribute(IXML_Document *doc, IXML_Element *parent,
+                                char *attrname, char *value)
 {
-	IXML_Attr *attr;
-	IXML_Node *child;
-	attr = ixmlDocument_createAttribute(doc, attrname);
-	child=ixmlDocument_createTextNode(doc, value);
-	ixmlNode_appendChild((IXML_Node *)attr,(IXML_Node *)child);
-	ixmlNode_appendChild((IXML_Node *)parent,(IXML_Node *)attr);
+	ixmlElement_setAttribute(parent, attrname, value);
 }
 
-static void add_value_element(IXML_Document *doc, IXML_Element *parent, char *tagname, char *value)
+static void add_value_element(IXML_Document *doc, IXML_Element *parent,
+                              char *tagname, char *value)
 {
 	IXML_Element *top;
 	IXML_Node *child;
@@ -77,14 +80,14 @@ static void add_value_element(IXML_Document *doc, IXML_Element *parent, char *ta
 
 	ixmlNode_appendChild((IXML_Node *)parent,(IXML_Node *)top);
 }
-static void add_value_element_int(IXML_Document *doc, IXML_Element *parent, char *tagname, int value)
+static void add_value_element_int(IXML_Document *doc, IXML_Element *parent,
+                                  char *tagname, int value)
 {
 	char *buf;
 
 	asprintf(&buf,"%d",value);
 	add_value_element(doc, parent, tagname, buf);
 	free(buf);
-	
 }
 
 static IXML_Element *gen_specversion(IXML_Document *doc, int major, int minor)
@@ -99,7 +102,9 @@ static IXML_Element *gen_specversion(IXML_Document *doc, int major, int minor)
 	return top;
 }
 
-static IXML_Element *gen_scpd_action(IXML_Document *doc, struct action *act, struct argument **arglist, const char **varnames)
+static IXML_Element *gen_scpd_action(IXML_Document *doc, struct action *act,
+                                     struct argument **arglist,
+                                     const char **varnames)
 {
 	IXML_Element *top;
 	IXML_Element *parent,*child;
@@ -123,7 +128,8 @@ static IXML_Element *gen_scpd_action(IXML_Document *doc, struct action *act, str
 	return top;
 }
 
-static IXML_Element *gen_scpd_actionlist(IXML_Document *doc, struct service *srv)
+static IXML_Element *gen_scpd_actionlist(IXML_Document *doc,
+                                         struct service *srv)
 {
 	IXML_Element *top;
 	IXML_Element *child;
@@ -204,6 +210,7 @@ IXML_Document *generate_scpd(struct service *srv)
 	doc = ixmlDocument_createDocument();
 
 	root=ixmlDocument_createElementNS(doc, "urn:schemas-upnp-org:service-1-0","scpd");
+	ixmlElement_setAttribute(root, "xmlns", "urn:schemas-upnp-org:service-1-0");
 	ixmlNode_appendChild((IXML_Node *)doc,(IXML_Node *)root);
 
 	child=gen_specversion(doc,1,0);
@@ -274,6 +281,7 @@ IXML_Document *generate_desc(void)
 	doc = ixmlDocument_createDocument();
 
 	root=ixmlDocument_createElementNS(doc, "urn:schemas-upnp-org:device-1-0","root");
+	ixmlElement_setAttribute(root, "xmlns", "urn:schemas-upnp-org:device-1-0");
 	ixmlNode_appendChild((IXML_Node *)doc,(IXML_Node *)root);
 	child=gen_specversion(doc,1,0);
 	ixmlNode_appendChild((IXML_Node *)root,(IXML_Node *)child);
@@ -305,10 +313,14 @@ int
 upnp_add_response(struct action_event *event, char *key, const char *value)
 {
 	char *val;
-	int res;
+	int rc;
+	int result = -1;
 
-	if (event->status)
-		return -1;
+	ENTER();
+
+	if (event->status) {
+		goto out;
+	}
 
 	val = strdup(value);
 	if (val == NULL) {
@@ -317,23 +329,26 @@ upnp_add_response(struct action_event *event, char *key, const char *value)
 		event->request->ActionResult = NULL;
 		event->request->ErrCode = UPNP_SOAP_E_ACTION_FAILED;
 		strcpy(event->request->ErrStr, strerror(errno));
-		return -1;
+		goto out;
 	}
 
-	res =
+	rc =
 	    UpnpAddToActionResponse(&event->request->ActionResult,
 				    event->request->ActionName,
 				    event->service->type, key, val);
-	if (res != UPNP_E_SUCCESS) {
+	if (rc != UPNP_E_SUCCESS) {
 		/* report custom error */
 		free(val);
 		event->request->ActionResult = NULL;
 		event->request->ErrCode = UPNP_SOAP_E_ACTION_FAILED;
-		strcpy(event->request->ErrStr, UpnpGetErrorMessage(res));
-		return -1;
+		strcpy(event->request->ErrStr, UpnpGetErrorMessage(rc));
+		goto out;
 	}
 
-	return 0;
+	result = 0;
+out:
+	LEAVE();
+	return result;
 }
 
 int upnp_append_variable(struct action_event *event,
@@ -341,13 +356,15 @@ int upnp_append_variable(struct action_event *event,
 {
 	char *value;
 	struct service *service = event->service;
-	int retval = 0;
+	int retval = -1;
+
+	ENTER();
 
 	if (varnum >= service->variable_count) {
 		upnp_set_error(event, UPNP_E_INTERNAL_ERROR,
 			       "Internal Error - illegal variable number %d",
 			       varnum);
-		return -1;
+		goto out;
 	}
 
 	ithread_mutex_lock(service->service_mutex);
@@ -356,12 +373,13 @@ int upnp_append_variable(struct action_event *event,
 	if (value == NULL) {
 		upnp_set_error(event, UPNP_E_INTERNAL_ERROR,
 			       "Internal Error");
-		retval = -1;
 	} else {
 		retval = upnp_add_response(event, paramname, value);
 	}
 
 	ithread_mutex_unlock(service->service_mutex);
+out:
+	LEAVE();
 	return retval;
 }
 
@@ -446,18 +464,23 @@ static struct action *find_action(struct service *event_service,
 	return NULL;
 }
 
-static int handle_subscription_request(struct
-						Upnp_Subscription_Request
-						*sr_event)
+static int handle_subscription_request(struct Upnp_Subscription_Request
+                                              *sr_event)
 {
-	struct service *subscription_service;
+	struct service *srv;
+	int i;
+	int eventVarCount = 0, eventVarIdx = 0;
+	const char **eventvar_names;
+	const char **eventvar_values;
+	ENTER();
+
 
 	printf("Subscription request\n");
 	printf("  %s\n", sr_event->UDN);
 	printf("  %s\n", sr_event->ServiceId);
 
-	subscription_service = find_service(sr_event->ServiceId);
-	if (subscription_service == NULL) {
+	srv = find_service(sr_event->ServiceId);
+	if (srv == NULL) {
 		fprintf(stderr, "%s: Unknown service '%s'\n", __FUNCTION__,
 			sr_event->ServiceId);
 		return -1;
@@ -465,16 +488,41 @@ static int handle_subscription_request(struct
 
 	ithread_mutex_lock(&device_mutex);
 
+	/* generate list of eventable variables */
+	for(i=0; i<srv->variable_count; i++) {
+		struct var_meta *metaEntry;
+		metaEntry = &(srv->variable_meta[i]);
+		if (metaEntry->sendevents == SENDEVENT_YES) {
+			eventVarCount++;
+		}
+	}
+	eventvar_names = malloc((eventVarCount+1) * sizeof(const char *));
+	eventvar_values = malloc((eventVarCount+1) * sizeof(const char *));
+	printf("%d evented variables\n", eventVarCount);
+
+	for(i=0; i<srv->variable_count; i++) {
+		struct var_meta *metaEntry;
+		metaEntry = &(srv->variable_meta[i]);
+		if (metaEntry->sendevents == SENDEVENT_YES) {
+			eventvar_names[eventVarIdx] = srv->variable_names[i];
+			eventvar_values[eventVarIdx] = srv->variable_values[i];
+			eventVarIdx++;
+		}
+	}
+	eventvar_names[eventVarIdx] = NULL;
+	eventvar_values[eventVarIdx] = NULL;
+
 	UpnpAcceptSubscription(device_handle,
 			       sr_event->UDN, sr_event->ServiceId,
-			       subscription_service->variable_names,
-			       (const char **) (subscription_service->
-						variable_values),
-			       subscription_service->variable_count,
+			       eventvar_names,
+			       eventvar_values,
+			       eventVarCount,
 			       sr_event->Sid);
 
 	ithread_mutex_unlock(&device_mutex);
 
+	LEAVE();
+
 	return 0;
 }
 
@@ -539,9 +587,10 @@ static int event_handler(Upnp_EventType EventType, void *event,
 		handle_action_request(event);
 		break;
 	case UPNP_CONTROL_GET_VAR_REQUEST:
-		printf("control get variable request\n");
+		printf("NOT IMPLEMENTED: control get variable request\n");
 		break;
 	case UPNP_EVENT_SUBSCRIPTION_REQUEST:
+		printf("event subscription request\n");
 		handle_subscription_request(event);
 		break;
 
@@ -571,16 +620,18 @@ int init_upnp(struct device *device_def)
 
 	/* register icons in web server */
         for (i=0; (icon_entry = upnp_device->icons[i]); i++) {
-		webserver_register_file(icon_entry->url);
+		webserver_register_file(icon_entry->url, "image/png");
         }
 
 	/* generate and register service schemas in web server */
         for (i=0; (srv = upnp_device->services[i]); i++) {
 		doc = generate_scpd(srv);
        		buf = ixmlDocumenttoString(doc);
-		webserver_register_buf(srv->scpd_url,buf);
+                printf("registering '%s'\n", srv->scpd_url);
+		webserver_register_buf(srv->scpd_url,buf,"text/xml");
 	}
 
+
 	rc = UpnpInit(ip_address, port);
 	if (UPNP_E_SUCCESS != rc) {
 		printf("UpnpInit() Error: %d\n", rc);
