@@ -21,6 +21,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -28,10 +32,13 @@
 #include <upnp/ithread.h>
 #include <upnp/upnptools.h>
 
+#include "logging.h"
+
 #include "upnp.h"
 #include "upnp_connmgr.h"
 
-#define CONNMGR_SERVICE "urn:upnp-org:serviceId:ConnectionManager"
+//#define CONNMGR_SERVICE "urn:upnp-org:serviceId:ConnectionManager"
+#define CONNMGR_SERVICE "urn:schemas-upnp-org:service:ConnectionManager"
 #define CONNMGR_TYPE	"urn:schemas-upnp-org:service:ConnectionManager:1"
 #define CONNMGR_SCPD_URL "/upnp/renderconnmgrSCPD.xml"
 #define CONNMGR_CONTROL_URL "/upnp/control/renderconnmgr1"
@@ -57,8 +64,8 @@ typedef enum {
 	CONNMGR_CMD_GETPROTOCOLINFO,           
 	CONNMGR_CMD_GETCURRENTCONNECTIONIDS,
 	CONNMGR_CMD_SETCURRENTCONNECTIONINFO,
-	CONNMGR_CMD_PREPAREFORCONNECTION,
-	CONNMGR_CMD_CONNECTIONCOMPLETE,
+	//CONNMGR_CMD_PREPAREFORCONNECTION,
+	//CONNMGR_CMD_CONNECTIONCOMPLETE,
 	CONNMGR_CMD_UNKNOWN,                  
 	CONNMGR_CMD_COUNT 
 } connmgr_cmd;
@@ -104,8 +111,8 @@ static struct argument **argument_list[] = {
 	[CONNMGR_CMD_GETPROTOCOLINFO] =			arguments_getprotocolinfo,           
 	[CONNMGR_CMD_GETCURRENTCONNECTIONIDS] =		arguments_getcurrentconnectionids,
 	[CONNMGR_CMD_SETCURRENTCONNECTIONINFO] =	arguments_setcurrentconnectioninfo,
-	[CONNMGR_CMD_PREPAREFORCONNECTION] =		arguments_prepareforconnection,
-	[CONNMGR_CMD_CONNECTIONCOMPLETE] =		arguments_connectioncomplete,
+	//[CONNMGR_CMD_PREPAREFORCONNECTION] =		arguments_prepareforconnection,
+	//[CONNMGR_CMD_CONNECTIONCOMPLETE] =		arguments_connectioncomplete,
 	[CONNMGR_CMD_UNKNOWN]	=	NULL
 };
 
@@ -128,9 +135,9 @@ static char *connmgr_values[] = {
 	[CONNMGR_VAR_SINK_PROTO_INFO] = "http-get:*:audio/mpeg:*",
 	[CONNMGR_VAR_CUR_CONN_IDS] = "0",
 	[CONNMGR_VAR_AAT_CONN_STATUS] = "Unknown",
-	[CONNMGR_VAR_AAT_CONN_MGR] = "",
-	[CONNMGR_VAR_AAT_DIR] = "Output",
-	[CONNMGR_VAR_AAT_PROTO_INFO] = "",
+	[CONNMGR_VAR_AAT_CONN_MGR] = "/",
+	[CONNMGR_VAR_AAT_DIR] = "Input",
+	[CONNMGR_VAR_AAT_PROTO_INFO] = ":::",
 	[CONNMGR_VAR_AAT_CONN_ID] = "-1",
 	[CONNMGR_VAR_AAT_AVT_ID] = "0",
 	[CONNMGR_VAR_AAT_RCS_ID] = "0",
@@ -179,9 +186,11 @@ static void register_mime_type_internal(const char *mime_type)
 	struct mime_type *entry;
 
 	for (entry = supported_types; entry; entry = entry->next) {
-		if (strcmp(entry->mime_type, mime_type) == 0)
+		if (strcmp(entry->mime_type, mime_type) == 0) {
 			return;
+		}
 	}
+	printf("Registering support for '%s'\n", mime_type);
 
 	entry = malloc(sizeof(struct mime_type));
 	entry->mime_type = strdup(mime_type);
@@ -191,8 +200,9 @@ static void register_mime_type_internal(const char *mime_type)
 void register_mime_type(const char *mime_type)
 {
 	register_mime_type_internal(mime_type);
-	if (strcmp("audio/mpeg", mime_type) == 0)
+	if (strcmp("audio/mpeg", mime_type) == 0) {
 		register_mime_type_internal("audio/x-mpeg");
+	}
 }
 
 int connmgr_init(void)
@@ -202,13 +212,16 @@ int connmgr_init(void)
 	char *p;
 	int offset;
 	int bufsize = 0;
+	int result = -1;
+
+	ENTER();
 
 	buf = malloc(bufsize);
 	p = buf;
 	if (buf == NULL) {
 		fprintf(stderr, "%s: initial malloc failed\n",
 			__FUNCTION__);
-		return -1;
+		goto out;
 	}
 
 	for (entry = supported_types; entry; entry = entry->next) {
@@ -218,7 +231,7 @@ int connmgr_init(void)
 		if (buf == NULL) {
 			fprintf(stderr, "%s: realloc failed\n",
 				__FUNCTION__);
-			return -1;
+			goto out;
 		}
 		p = buf;
 		p += offset;
@@ -236,23 +249,40 @@ int connmgr_init(void)
 	*p = '\0';
 
 	connmgr_values[CONNMGR_VAR_SINK_PROTO_INFO] = buf;
-	return 0;
+	//connmgr_values[CONNMGR_VAR_SRC_PROTO_INFO] = buf;
+
+	result = 0;
+out:
+	LEAVE();
+	return result;
 }
 
 
 static int get_protocol_info(struct action_event *event)
 {
-	printf("%s: ENTER\n", __FUNCTION__);
-	upnp_add_response(event, "Source", "");
+	ENTER();
+	upnp_append_variable(event, CONNMGR_VAR_SRC_PROTO_INFO, "Source");
 	upnp_append_variable(event, CONNMGR_VAR_SINK_PROTO_INFO, "Sink");
+	LEAVE();
 	return event->status;
 }
 
 static int get_current_conn_ids(struct action_event *event)
 {
-	printf("%s: ENTER\n", __FUNCTION__);
-	return upnp_append_variable(event, CONNMGR_VAR_CUR_CONN_IDS,
-				    "ConnectionIDs");
+	int rc;
+	ENTER();
+	upnp_add_response(event, "ConnectionIDs", "0");
+	///rc = upnp_append_variable(event, CONNMGR_VAR_CUR_CONN_IDS,
+	//			    "ConnectionIDs");
+	LEAVE();
+	return rc;
+}
+
+static int prepare_for_connection(struct action_event *event)
+{
+	ENTER();
+	LEAVE();
+	return 0;
 }
 
 static int get_current_conn_info(struct action_event *event)
@@ -260,7 +290,7 @@ static int get_current_conn_info(struct action_event *event)
 	int rc;
 	char *value;
 
-	printf("%s: ENTER\n", __FUNCTION__);
+	ENTER();
 	value = upnp_get_string(event, "ConnectionID");
 	if (value == NULL) {
 		rc = -1;
@@ -297,9 +327,8 @@ static int get_current_conn_info(struct action_event *event)
 		goto out;
 
       out:
+	LEAVE();
 	return rc;
-
-
 }
 
 
@@ -307,8 +336,8 @@ static struct action connmgr_actions[] = {
 	[CONNMGR_CMD_GETPROTOCOLINFO] =		{"GetProtocolInfo", get_protocol_info},
 	[CONNMGR_CMD_GETCURRENTCONNECTIONIDS] =	{"GetCurrentConnectionIDs", get_current_conn_ids},
 	[CONNMGR_CMD_SETCURRENTCONNECTIONINFO] ={"GetCurrentConnectionInfo", get_current_conn_info},
-	[CONNMGR_CMD_PREPAREFORCONNECTION] =	{"PrepareForConnection", NULL},	/* optional */
-	[CONNMGR_CMD_CONNECTIONCOMPLETE] =	{"ConnectionComplete", NULL},	/* optional */
+	//[CONNMGR_CMD_PREPAREFORCONNECTION] =	{"PrepareForConnection", prepare_for_connection}, /* optional */
+	//[CONNMGR_CMD_CONNECTIONCOMPLETE] =	{"ConnectionComplete", NULL},	/* optional */
 	[CONNMGR_CMD_UNKNOWN] =			{NULL, NULL}
 };
 
