@@ -21,6 +21,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -32,7 +36,8 @@
 #include "upnp.h"
 #include "upnp_control.h"
 
-#define CONTROL_SERVICE "urn:upnp-org:serviceId:RenderingControl"
+//#define CONTROL_SERVICE "urn:upnp-org:serviceId:RenderingControl"
+#define CONTROL_SERVICE "urn:schemas-upnp-org:service:RenderingControl"
 #define CONTROL_TYPE "urn:schemas-upnp-org:service:RenderingControl:1"
 #define CONTROL_SCPD_URL "/upnp/rendercontrolSCPD.xml"
 #define CONTROL_CONTROL_URL "/upnp/control/rendercontrol1"
@@ -194,7 +199,7 @@ static struct var_meta control_var_meta[] = {
 };
 
 static char *control_values[] = {
-	[CONTROL_VAR_LAST_CHANGE] = "",
+	[CONTROL_VAR_LAST_CHANGE] = "<Event xmlns = \"urn:schemas-upnp-org:metadata-1-0/AVT/\"><InstanceID val=\"0\"></InstanceID></Event>",
 	[CONTROL_VAR_PRESET_NAME_LIST] = "",
 	[CONTROL_VAR_AAT_CHANNEL] = "",
 	[CONTROL_VAR_AAT_INSTANCE_ID] = "0",
