@@ -21,14 +21,15 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #define _GNU_SOURCE
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <upnp/upnp.h>
-#include <upnp/ithread.h>
-#include <upnp/upnptools.h>
 #include <errno.h>
 #include <stdarg.h>
 
@@ -36,8 +37,11 @@
 #include <sys/socket.h>
 #include <arpa/inet.h>
 
-#include "../config.h"
+#include <upnp/upnp.h>
+#include <upnp/ithread.h>
+#include <upnp/upnptools.h>
 
+#include "logging.h"
 #include "webserver.h"
 #include "upnp.h"
 #include "upnp_renderer.h"
@@ -93,5 +97,21 @@ static struct device render_device = {
 
 int init_upnp_renderer(void)
 {
-	return init_upnp(&render_device);
+	int result = -1;
+	int rc;
+
+	ENTER();
+
+        rc = connmgr_init();
+	if (rc != 0) {
+		goto out;
+	}
+	rc = init_upnp(&render_device);
+	if (rc != 0) {
+		goto out;
+	}
+	result = 0;
+out:
+	LEAVE();
+	return result;
 }
