@@ -21,19 +21,30 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+
+#include <glib.h>
+
 #include <upnp/upnp.h>
 #include <upnp/ithread.h>
 #include <upnp/upnptools.h>
 
+
+#include "logging.h"
+
 #include "upnp.h"
 #include "upnp_transport.h"
 #include "output_gstreamer.h"
 
-#define TRANSPORT_SERVICE "urn:upnp-org:serviceId:AVTransport"
+//#define TRANSPORT_SERVICE "urn:upnp-org:serviceId:AVTransport"
+#define TRANSPORT_SERVICE "urn:schemas-upnp-org:service:AVTransport"
 #define TRANSPORT_TYPE "urn:schemas-upnp-org:service:AVTransport:1"
 #define TRANSPORT_SCPD_URL "/upnp/rendertransportSCPD.xml"
 #define TRANSPORT_CONTROL_URL "/upnp/control/rendertransport1"
@@ -145,7 +156,7 @@ static char *transport_values[] = {
 	[TRANSPORT_VAR_POS_REC_QUAL_MODE] = "NOT_IMPLEMENTED",
 	[TRANSPORT_VAR_NR_TRACKS] = "0",
 	[TRANSPORT_VAR_CUR_TRACK] = "0",
-	[TRANSPORT_VAR_CUR_TRACK_DUR] = "",
+	[TRANSPORT_VAR_CUR_TRACK_DUR] = "00:00:00",
 	[TRANSPORT_VAR_CUR_MEDIA_DUR] = "",
 	[TRANSPORT_VAR_CUR_TRACK_META] = "",
 	[TRANSPORT_VAR_CUR_TRACK_URI] = "",
@@ -153,11 +164,12 @@ static char *transport_values[] = {
 	[TRANSPORT_VAR_AV_URI_META] = "",
 	[TRANSPORT_VAR_NEXT_AV_URI] = "",
 	[TRANSPORT_VAR_NEXT_AV_URI_META] = "",
-	[TRANSPORT_VAR_REL_TIME_POS] = "",
-	[TRANSPORT_VAR_ABS_TIME_POS] = "",
-	[TRANSPORT_VAR_REL_CTR_POS] = "0",
-	[TRANSPORT_VAR_ABS_CTR_POS] = "0",
-	[TRANSPORT_VAR_LAST_CHANGE] = "",
+	[TRANSPORT_VAR_REL_TIME_POS] = "NOT_IMPLEMENTED",
+	[TRANSPORT_VAR_ABS_TIME_POS] = "NOT_IMPLEMENTED",
+	[TRANSPORT_VAR_REL_CTR_POS] = "2147483647",
+	[TRANSPORT_VAR_ABS_CTR_POS] = "2147483647",
+        [TRANSPORT_VAR_LAST_CHANGE] = "<Event xmlns = \"urn:schemas-upnp-org:metadata-1-0/AVT/\"><InstanceID val=\"0\"></InstanceID></Event>",
+
 	[TRANSPORT_VAR_AAT_SEEK_MODE] = "TRACK_NR",
 	[TRANSPORT_VAR_AAT_SEEK_TARGET] = "",
 	[TRANSPORT_VAR_AAT_INSTANCE_ID] = "0",
@@ -456,7 +468,7 @@ static int get_media_info(struct action_event *event)
 {
 	char *value;
 	int rc;
-	printf("%s: ENTER\n", __FUNCTION__);
+	ENTER();
 
 	value = upnp_get_string(event, "InstanceID");
 	if (value == NULL) {
@@ -516,8 +528,7 @@ static int get_media_info(struct action_event *event)
 	return rc;
 }
 
-
-void sgmlescape(const char *str, char *target, uint32_t * length)
+static void sgmlattrescape(const char *str, char *target, uint32_t * length)
 {
 	if (target != NULL) {
 		uint32_t len = 0;
@@ -526,6 +537,9 @@ void sgmlescape(const char *str, char *target, uint32_t * length)
 			if (*str == '<') {
 				memcpy(target + len, "&lt;", 4);
 				len += 4;
+			} else if (*str == '"') {
+				memcpy(target + len, "%22", 3);
+				len += 3;
 			} else if (*str == '>') {
 				memcpy(target + len, "&gt;", 4);
 				len += 4;
@@ -546,6 +560,8 @@ void sgmlescape(const char *str, char *target, uint32_t * length)
 		for (; *str; str++) {
 			if (*str == '<') {
 				len += 4;
+			} else if (*str == '"') {
+				len += 3;
 			} else if (*str == '>') {
 				len += 4;
 			} else if (*str == '&') {
@@ -559,14 +575,14 @@ void sgmlescape(const char *str, char *target, uint32_t * length)
 	}
 }
 
-char *xsgmlescape(const char *str)
+static char *xsgmlattrescape(const char *str)
 {
 	uint32_t len;
 	char *out;
 
-	sgmlescape(str, NULL, &len);
+	sgmlattrescape(str, NULL, &len);
 	out = malloc(len + 1);
-	sgmlescape(str, out, NULL);
+	sgmlattrescape(str, out, NULL);
 	return out;
 }
 
@@ -582,8 +598,11 @@ static void notify_lastchange(struct action_event *event, char *value)
 
 
 	printf("Event: '%s'\n", value);
-	varvalues[0] = xsgmlescape(value);
+	//varvalues[0] = xsgmlescape(value);
+	varvalues[0] = value;
 
+
+	transport_values[TRANSPORT_VAR_LAST_CHANGE] = value;
 	UpnpNotify(device_handle, event->request->DevUDN,
 		   event->request->ServiceID, varnames,
 		   (const char **) varvalues, 1);
@@ -596,10 +615,17 @@ static void change_var(struct action_event *event, int varnum,
 		       char *new_value)
 {
 	char *buf;
-	if ((varnum < 0) || (varnum >= TRANSPORT_VAR_UNKNOWN))
+
+	ENTER();
+
+	if ((varnum < 0) || (varnum >= TRANSPORT_VAR_UNKNOWN)) {
+		LEAVE();
 		return;
-	if (new_value == NULL)
+	}
+	if (new_value == NULL) {
+		LEAVE();
 		return;
+	}
 
 	//if (transport_values[varnum]) {
 	//      free(transport_values[varnum]);
@@ -607,10 +633,11 @@ static void change_var(struct action_event *event, int varnum,
 	transport_values[varnum] = strdup(new_value);
 	asprintf(&buf,
 		 "<Event xmlns = \"urn:schemas-upnp-org:metadata-1-0/AVT/\"><InstanceID val=\"0\"><%s val=\"%s\"/></InstanceID></Event>",
-		 transport_variables[varnum], transport_values[varnum]);
+		 transport_variables[varnum], xsgmlattrescape(transport_values[varnum]));
 	notify_lastchange(event, buf);
 	//free(buf)
 
+	LEAVE();
 
 	return;
 }
@@ -618,7 +645,9 @@ static void change_var(struct action_event *event, int varnum,
 static int obtain_instanceid(struct action_event *event, int *instance)
 {
 	char *value;
-	int retval = 0;
+	int rc = 0;
+	
+	ENTER();
 
 	value = upnp_get_string(event, "InstanceID");
 	if (value == NULL) {
@@ -631,7 +660,9 @@ static int obtain_instanceid(struct action_event *event, int *instance)
 
 	// TODO - parse value, and store in *instance, if instance!=NULL
 
-	return retval;
+	LEAVE();
+
+	return rc;
 }
 
 /* UPnP action handlers */
@@ -639,15 +670,17 @@ static int obtain_instanceid(struct action_event *event, int *instance)
 static int set_avtransport_uri(struct action_event *event)
 {
 	char *value;
-	int retval = 0;
+	int rc = 0;
 
-	printf("%s ENTER\n", __FUNCTION__);
+	ENTER();
 
 	if (obtain_instanceid(event, NULL)) {
+		LEAVE();
 		return -1;
 	}
 	value = upnp_get_string(event, "CurrentURI");
 	if (value == NULL) {
+		LEAVE();
 		return -1;
 	}
 
@@ -663,7 +696,7 @@ static int set_avtransport_uri(struct action_event *event)
 
 	value = upnp_get_string(event, "CurrentURIMetaData");
 	if (value == NULL) {
-		retval = -1;
+		rc = -1;
 	} else {
 		printf("%s: CurrentURIMetaData='%s'\n", __FUNCTION__,
 		       value);
@@ -673,40 +706,43 @@ static int set_avtransport_uri(struct action_event *event)
 
 	ithread_mutex_unlock(&transport_mutex);
 
-	printf("%s LEAVE\n", __FUNCTION__);
-	return retval;
+	LEAVE();
+	return rc;
 }
 
 static int set_next_avtransport_uri(struct action_event *event)
 {
 	char *value;
 
-	printf("%s ENTER\n", __FUNCTION__);
+	ENTER();
 
 	if (obtain_instanceid(event, NULL)) {
+		LEAVE();
 		return -1;
 	}
 	value = upnp_get_string(event, "NextURI");
 	if (value == NULL) {
+		LEAVE();
 		return -1;
 	}
 	printf("%s: NextURI='%s'\n", __FUNCTION__, value);
 	free(value);
 	value = upnp_get_string(event, "NextURIMetaData");
 	if (value == NULL) {
+		LEAVE();
 		return -1;
 	}
 	printf("%s: NextURIMetaData='%s'\n", __FUNCTION__, value);
 	free(value);
 
-	printf("%s LEAVE\n", __FUNCTION__);
+	LEAVE();
 	return 0;
 }
 
 static int get_transport_info(struct action_event *event)
 {
 	int rc;
-	printf("%s: ENTER\n", __FUNCTION__);
+	ENTER();
 
 	if (obtain_instanceid(event, NULL)) {
 		rc = -1;
@@ -730,13 +766,14 @@ static int get_transport_info(struct action_event *event)
 		goto out;
 
       out:
+	LEAVE();
 	return rc;
 }
 
 static int get_transport_settings(struct action_event *event)
 {
 	int rc = 0;
-	printf("%s: ENTER\n", __FUNCTION__);
+	ENTER();
 
 	if (obtain_instanceid(event, NULL)) {
 		rc = -1;
@@ -744,13 +781,14 @@ static int get_transport_settings(struct action_event *event)
 	}
 
       out:
+	LEAVE();
 	return rc;
 }
 
 static int get_position_info(struct action_event *event)
 {
 	int rc;
-	printf("%s: ENTER\n", __FUNCTION__);
+	ENTER();
 
 	if (obtain_instanceid(event, NULL)) {
 		rc = -1;
@@ -797,13 +835,14 @@ static int get_position_info(struct action_event *event)
 		goto out;
 
       out:
+	LEAVE();
 	return rc;
 }
 
 static int get_device_caps(struct action_event *event)
 {
 	int rc = 0;
-	printf("%s: ENTER\n", __FUNCTION__);
+	ENTER();
 
 	if (obtain_instanceid(event, NULL)) {
 		rc = -1;
@@ -811,12 +850,13 @@ static int get_device_caps(struct action_event *event)
 	}
 
       out:
+	LEAVE();
 	return rc;
 }
 
 static int stop(struct action_event *event)
 {
-	printf("%s: ENTER\n", __FUNCTION__);
+	ENTER();
 
 	if (obtain_instanceid(event, NULL)) {
 		return -1;
@@ -847,7 +887,7 @@ static int stop(struct action_event *event)
 	}
 	ithread_mutex_unlock(&transport_mutex);
 
-	printf("%s: LEAVE\n", __FUNCTION__);
+	LEAVE();
 
 	return 0;
 }
@@ -855,11 +895,12 @@ static int stop(struct action_event *event)
 
 static int play(struct action_event *event)
 {
-	int retval = 0;
-	printf("%s: ENTER\n", __FUNCTION__);
+	int rc = 0;
 
+	ENTER();
 
 	if (obtain_instanceid(event, NULL)) {
+		LEAVE();
 		return -1;
 	}
 
@@ -872,7 +913,7 @@ static int play(struct action_event *event)
 	case TRANSPORT_PAUSED_PLAYBACK:
 		if (output_play()) {
 			upnp_set_error(event, 704, "Playing failed");
-			retval = -1;
+			rc = -1;
 		} else {
 			transport_state = TRANSPORT_PLAYING;
 			change_var(event, TRANSPORT_VAR_TRANSPORT_STATE,
@@ -889,52 +930,55 @@ static int play(struct action_event *event)
 		/* action not allowed in these states - error 701 */
 		upnp_set_error(event, UPNP_TRANSPORT_E_TRANSITION_NA,
 			       "Transition not allowed");
-		retval = -1;
+		rc = -1;
 
 		break;
 	}
 	ithread_mutex_unlock(&transport_mutex);
 
-	printf("%s: LEAVE\n", __FUNCTION__);
+	LEAVE();
 
-	return retval;
+	return rc;
 }
 
 static int seek(struct action_event *event)
 {
-	printf("%s: ENTER\n", __FUNCTION__);
+	int rc = 0;
+	ENTER();
 
 	if (obtain_instanceid(event, NULL)) {
-		return -1;
+		rc = -1;
 	}
 
-	printf("%s: LEAVE\n", __FUNCTION__);
+	LEAVE();
 
-	return 0;
+	return rc;
 }
 
 static int next(struct action_event *event)
 {
-	printf("%s: ENTER\n", __FUNCTION__);
+	int rc = 0;
+
+	ENTER();
 
 	if (obtain_instanceid(event, NULL)) {
-		return -1;
+		rc = -1;
 	}
 
-	printf("%s: LEAVE\n", __FUNCTION__);
+	LEAVE();
 
-	return 0;
+	return rc;
 }
 
 static int previous(struct action_event *event)
 {
-	printf("%s: ENTER\n", __FUNCTION__);
+	ENTER();
 
 	if (obtain_instanceid(event, NULL)) {
 		return -1;
 	}
 
-	printf("%s: LEAVE\n", __FUNCTION__);
+	LEAVE();
 
 	return 0;
 }
