@@ -21,18 +21,26 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <errno.h>
+#include <error.h>
 
 #include <upnp/upnp.h>
 #include <upnp/ithread.h>
-#include <errno.h>
+
+#include "logging.h"
+#include "webserver.h"
 
 typedef struct {
-	size_t pos;
+	off_t pos;
 	const char *contents;
 	size_t len;
 } WebServerFile;
@@ -42,43 +50,57 @@ struct virtual_file;
 static struct virtual_file {
 	const char *virtual_fname;
 	const char *contents;
+	const char *content_type;
 	size_t len;
 	struct virtual_file *next;
 } *virtual_files = NULL;
 
-int webserver_register_buf(const char *path, const char *contents)
+int webserver_register_buf(const char *path, const char *contents, const char *content_type)
 {
+	int result = -1;
 	struct virtual_file *entry;
+
+	ENTER();
+
 	entry = malloc(sizeof(struct virtual_file));
 	if (entry == NULL) {
-		return -1;
+		goto out;
 	}
 	entry->len = strlen(contents);
 	entry->contents = contents;
 	entry->virtual_fname = path;
+	entry->content_type = content_type;
 	entry->next = virtual_files;
 	virtual_files = entry;
-	return 0;
+	result = 0;
+
+out:
+	LEAVE();
+	return result;
 }
 
-int webserver_register_file(const char *path)
+int webserver_register_file(const char *path, const char *content_type)
 {
-	char *local_fname;
+	char local_fname[PATH_MAX];
 	struct stat buf;
 	struct virtual_file *entry;
 	int rc;
+	int result = -1;
 
-	local_fname = strrchr(path, '/') + 1;
+	ENTER();
+
+	snprintf(local_fname, PATH_MAX, "%s%s", PKG_DATADIR,
+	         strrchr(path, '/'));
 
 	rc = stat(local_fname, &buf);
 	if (rc) {
-		perror("Stat failed");
-		return -1;
+		error(0, errno, "Could not stat '%s'", local_fname);
+		goto out;
 	}
 
 	entry = malloc(sizeof(struct virtual_file));
 	if (entry == NULL) {
-		return -1;
+		goto out;
 	}
 	if (buf.st_size) {
 		char *cbuf;
@@ -86,12 +108,12 @@ int webserver_register_file(const char *path)
 		in = fopen(local_fname, "r");
 		if (in == NULL) {
 			free(entry);
-			return -1;
+			goto out;
 		}
 		cbuf = malloc(buf.st_size);
 		if (cbuf == NULL) {
 			free(entry);
-			return -1;
+			goto out;
 		}
 		fread(cbuf, buf.st_size, 1, in);
 		fclose(in);
@@ -103,17 +125,24 @@ int webserver_register_file(const char *path)
 		entry->contents = NULL;
 	}
 	entry->virtual_fname = path;
+	entry->content_type = content_type;
 	entry->next = virtual_files;
 	virtual_files = entry;
-	return 0;
+	result = 0;
+out:
+	LEAVE();
+	return result;
 }
 
 static int webserver_get_info(const char *filename, struct File_Info *info)
 {
+	int result = -1;
 	struct virtual_file *virtfile = virtual_files;
-	fprintf(stderr, "%s:(filename='%s',info)\n", __FUNCTION__,
-		filename);
 
+	ENTER();
+
+	fprintf(stderr, "%s:(filename='%s',info=%p)\n", __FUNCTION__,
+		filename, info);
 
 	while (virtfile != NULL) {
 		if (strcmp(filename, virtfile->virtual_fname) == 0) {
@@ -122,25 +151,30 @@ static int webserver_get_info(const char *filename, struct File_Info *info)
 			info->is_directory = 0;
 			info->is_readable = 1;
 			info->content_type =
-			    ixmlCloneDOMString("text/xml");
-			return 0;
+			    ixmlCloneDOMString(virtfile->content_type);
+			result = 0;
+			goto out;
 		}
 		virtfile = virtfile->next;
 	}
-	return -1;
+        printf("Not found\n");
+out:
+	LEAVE();
+	return result;
 }
 
 static UpnpWebFileHandle
 webserver_open(const char *filename, enum UpnpOpenFileMode mode)
 {
 	struct virtual_file *virtfile = virtual_files;
-	WebServerFile *file;
+	WebServerFile *file = NULL;
 
+	ENTER();
 	if (mode != UPNP_READ) {
 		fprintf(stderr,
 			"%s: ignoring request to open file for writing\n",
 			filename);
-		return NULL;
+		goto out;
 	}
 
 	while (virtfile != NULL) {
@@ -149,15 +183,17 @@ webserver_open(const char *filename, enum UpnpOpenFileMode mode)
 			file->pos = 0;
 			file->len = virtfile->len;
 			file->contents = virtfile->contents;
-			return file;
+			goto out;
 		}
 		virtfile = virtfile->next;
 	}
 
-	return NULL;
+out:
+	LEAVE();
+	return file;
 }
 
-static inline int min(int a, int b)
+static inline int minimum(int a, int b)
 {
 	return (a<b)?a:b;
 }
@@ -167,63 +203,69 @@ static int webserver_read(UpnpWebFileHandle fh, char *buf, size_t buflen)
 	WebServerFile *file = (WebServerFile *) fh;
 	ssize_t len = -1;
 
-	len = min(buflen, file->len - file->pos);
+	ENTER();
+
+	len = minimum(buflen, file->len - file->pos);
 	memcpy(buf, file->contents + file->pos, len);
 
-	if (len < 0)
-		fprintf(stderr, "%s: cannot read: %s\n", __FUNCTION__,
-			strerror(errno));
-	else
+	if (len < 0) {
+		error(0, errno, "%s failed", __FUNCTION__);
+	} else {
 		file->pos += len;
+	}
 
+	LEAVE();
 	return len;
 }
 
 static int webserver_write(UpnpWebFileHandle fh, char *buf, size_t buflen)
 {
+	ENTER();
+	LEAVE();
 	return -1;
 }
 
-static int webserver_seek(UpnpWebFileHandle fh, long offset, int origin)
+static int webserver_seek(UpnpWebFileHandle fh, off_t offset, int origin)
 {
 	WebServerFile *file = (WebServerFile *) fh;
-	long newpos = -1;
+	off_t newpos = -1;
+	int result = -1;
+	
+	ENTER();
 
 	switch (origin) {
 	case SEEK_SET:
-		fprintf(stderr, "Attempting to seek to %ld (was at %d)\n",
-			offset, file->pos);
 		newpos = offset;
 		break;
 	case SEEK_CUR:
-		fprintf(stderr, "Attempting to seek by %ld from %d\n",
-			offset, file->pos);
 		newpos = file->pos + offset;
 		break;
 	case SEEK_END:
-		fprintf(stderr,
-			"Attempting to seek by %ld from end (was at %d)\n",
-			offset, file->pos);
 		newpos = file->len + offset;
 		break;
 	}
 
 	if (newpos < 0 || newpos > file->len) {
-		fprintf(stderr, "%s: cannot seek: %s\n", __FUNCTION__,
-			strerror(EINVAL));
-		return -1;
+		error(0, errno, "%s seek failed", __FUNCTION__);
+		goto out;
 	}
 
 	file->pos = newpos;
-	return 0;
+	result = 0;
+out:
+	LEAVE();
+	return result;
 }
 
 static int webserver_close(UpnpWebFileHandle fh)
 {
 	WebServerFile *file = (WebServerFile *) fh;
 
+	ENTER();
+
 	free(file);
 
+	LEAVE();
 	return 0;
 }
 
