@@ -25,7 +25,9 @@
 #define _WEBSERVER_H
 
 extern struct UpnpVirtualDirCallbacks virtual_dir_callbacks;
-extern int webserver_register_buf(const char *path, const char *contents);
-extern int webserver_register_file(const char *path);
+extern int webserver_register_buf(const char *path, const char *contents,
+                                  const char *content_type);
+extern int webserver_register_file(const char *path,
+                                   const char *content_type);
 
 #endif /* _WEBSERVER_H */
